"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var _ = require("lodash");
var Mustache = require("mustache");
function BuildNameGenerator(cookbook) {
    function getRecipe(recipeKey) {
        return cookbook.recipes[recipeKey];
    }
    function getDefaultRecipeKey() {
        if (cookbook.recipeOrder) {
            return cookbook.recipeOrder[0];
        }
        if (cookbook.recipes.base) {
            return "base";
        }
        return Object.keys(cookbook.recipes)[0];
    }
    function getDefaultVariantKey(recipeKey) {
        var recipe = getRecipe(recipeKey);
        if (recipe.variantOrder) {
            return recipe.variantOrder[0];
        }
        if (recipe.variants.base) {
            return "base";
        }
        return Object.keys(recipe.variants)[0];
    }
    function getDefaultKeys() {
        var recipeKey = getDefaultRecipeKey();
        return {
            recipeKey: recipeKey,
            variantKey: getDefaultVariantKey(recipeKey)
        };
    }
    function recipesForDisplay() {
        return Object.keys(cookbook.recipes).reduce(function (acc, key) {
            var recipe = getRecipe(key);
            if (recipe.display != "") {
                acc.push({ key: key, display: recipe.display });
            }
            return acc;
        }, []);
    }
    function variantsForRecipe(recipeKey) {
        var variants = getRecipe(recipeKey).variants;
        return Object.keys(variants).reduce(function (acc, key) {
            var variant = variants[key];
            if (variant.display != "") {
                acc.push({ key: key, display: variant.display });
            }
            return acc;
        }, []);
    }
    // keeps a shuffled list of ingredients so we don't get too many duplicates.
    var cachedIngredients = {};
    var cachedIngredientsThreshold = {};
    // this function is a bit complicated because we support two different formats
    // one with variants
    function fetchIngredient(ingredientKey) {
        var cache = cachedIngredients[ingredientKey];
        if (cache && cache.length > cachedIngredientsThreshold[ingredientKey]) {
            return cache.shift();
        }
        var m = ingredientKey.match(/(.+)\$(.+)/);
        var recipeKey = ingredientKey;
        var variantKey = "base";
        if (m) {
            recipeKey = m[1];
            variantKey = m[2];
        }
        var recipe = getRecipe(recipeKey);
        var ingredients = recipe.variants[variantKey].ingredients;
        if (!ingredients) {
            throw new Error("Ingredient specified by key " + recipeKey + " not found!");
        }
        if (process.env.EVERY_NAME_FIX_SHUFFLE == "true") {
            cachedIngredients[ingredientKey] = _.reverse(_.clone(ingredients));
        }
        else {
            cachedIngredients[ingredientKey] = _.shuffle(_.clone(ingredients));
        }
        cachedIngredientsThreshold[ingredientKey] = ingredients.length >= 5 ? 1 : 0;
        return cachedIngredients[ingredientKey].shift();
    }
    var cachedRecipeFunctionHashInitialized = false;
    var cachedRecipeFunctionHash = {};
    function recipeFunctionHash() {
        if (cachedRecipeFunctionHashInitialized) {
            return cachedRecipeFunctionHash;
        }
        var list = {};
        Object.keys(cookbook.recipes).forEach(function (key) {
            var recipe = cookbook.recipes[key];
            Object.keys(recipe.variants).forEach(function (variant) {
                var keyName = key + "$" + variant;
                var func = function () { return makeName(keyName); }; // eslint-disable-line @typescript-eslint/no-use-before-define
                list[keyName] = func;
                if (variant === "base") {
                    list[key] = func;
                }
            });
        });
        cachedRecipeFunctionHash = list;
        cachedRecipeFunctionHashInitialized = true;
        return cachedRecipeFunctionHash;
    }
    function makeName(ingredientKey) {
        return Mustache.render(fetchIngredient(ingredientKey), recipeFunctionHash());
    }
    function makeNames(ingredientKey, count) {
        return _.times(count, function () { return makeName(ingredientKey); });
    }
    return {
        getRecipe: getRecipe,
        getDefaultKeys: getDefaultKeys,
        getDefaultRecipeKey: getDefaultRecipeKey,
        getDefaultVariantKey: getDefaultVariantKey,
        recipesForDisplay: recipesForDisplay,
        variantsForRecipe: variantsForRecipe,
        makeName: makeName,
        makeNames: makeNames
    };
}
exports.BuildNameGenerator = BuildNameGenerator;
//# sourceMappingURL=index.js.map